@@ -1,4 +1,5 @@
 # Revisions
+# June 2025 found & fixed bugs largely related to handling various forms of zero
 # feb 2024 allow "binSize" to prepend zeroes to any base (as string)
 # June-July 2023 fixed bug related to log() fun returning floating-point precision
 #   errors that caused floor() to be too small. 
@@ -25,7 +26,6 @@ if(!(2<=frombase && 36 >= frombase && 2<=tobase && 36>= tobase)){
 	stop('Both bases must be in range 2:36')
 }
 classOut <- match.arg(classOut)
-# browser()
 allChar <- c(0:9,letters)# validate by checking allChar[1:frombase]
 theans <-list()  
 inclass <- class(x)
@@ -55,10 +55,6 @@ for (jl in 1:length(x) ) {
 	isPos = TRUE 
 # nonlists are happy being indexed with [[k]][1] as are charstrings 
 # kill off zeros and vals < 1,  like 1.23e-4 
-	if( !is.na(suppressWarnings(as.numeric(x[[jl]])) ) && abs(as.numeric(x[[jl]]) ) < 1){
-		theans[[jl]] <- '0'
-		next
-	}
 	if (is.numeric(x[[jl]]) || gmp::is.bigz(x[[jl]])){
 # remove decimal part
 		if(is.numeric(x[[jl]])){
@@ -126,14 +122,12 @@ for (jl in 1:length(x) ) {
 # but if decimal and NO e, take floor()
 #subtle bug: if idiot entered a hex, e.g.  '3e4', this if() will catch it.  Best I can do is make sure there's at least one char BEFORE an "e"
 # Fixed 29aug2023
-#browser()
 	if(frombase == 10 && length(grep('e',xchar)) && grep('e',xtmp)>1 ) {
 				xchar <- noExp(xchar)
 				xtmp <- unlist(strsplit(xchar,split=''))
 	#  don't  make x[[jl]] numeric as that buggers bigz, for one thing. 
 			} else 	if(length(grep(paste0('[',thedec,']'), xchar) ) ) {
-	# looks like bug here - if it's xxx.(anything),that {,} requires a first zero {0,}
-#				xchar <- gsub(paste0('[',thedec,'].{,}$'),'',xchar)
+# if it's xxx.(anything),that {,} requires a first zero {0,}
 				xchar <- gsub(paste0('[',thedec,'].{0,}$'),'',xchar)
 				warning('data to right of decimal removed. Failure may occur.')
 				xtmp <- unlist(strsplit(xchar,split=''))	
@@ -156,13 +150,19 @@ for (jl in 1:length(x) ) {
 	maxpow <-frombase^length(xtmp) 
 	if(maxpow < 2147483647) {   #2^31-1
 #  strtoi won't overflow **because** I limited size of input to this 'if'	
-#HOWEVER: stroi will "kill off" the lead zero if input is 2s comp	
+#HOWEVER: stroi will "kill off" the lead zero if input is 2s comp
+# TODO: catch NA which can occur if a UTF8 char snuck into the input x	
 		foo <- strtoi(xchar,frombase)
-# same fix for precision rounding errors as the "else" below		
+#New June 2025: catch zero values here
+# same fix for precision rounding errors as the "else" below
+		if(foo > 0) {		
 		powM <- floor(log(foo,tobase)) +1  # how many places needed
-if ( !(length(powM)) || !is.numeric(powM) || powM < 0) browser()
 
 		mout <- rep('0',powM+1) 
+		} else {
+			powM = 0
+			mout <- rep('0',times = max(16,binSize)+1)
+			}
 	#remember leftmost element of vector is index 1
 #when powM = 0 this would cause 'trouble'
 		if (powM > 0) {
@@ -186,7 +186,6 @@ if ( !(length(powM)) || !is.numeric(powM) || powM < 0) browser()
 		} 
 	} else {
 #big number so use gmp and do things the hard way,loop over x terms
-# browser()  # bug -- somehow my mpfr input failed to get the decimal part removed???
 			foo <- gmp::as.bigz(0)   
 			xrev <- rev(xtmp)
 			for (jg in length(xtmp):1){
@@ -202,10 +201,15 @@ if ( !(length(powM)) || !is.numeric(powM) || powM < 0) browser()
 			    }
 	# powM is numeric even when foo is bigz
 			gtobase <- gmp::as.bigz(tobase)
+# Fix same bug as above: if input is lots of zeros, need to adjust powM			
 # foo == 2^62 returns a log 61.99999..fix by just adding one
-			powM <- floor(log(foo,gtobase)) + 1 # how many places will I need
-if (!length(powM) || !is.numeric(powM) || powM < 0) browser()
-			mout <- rep('0',powM+1) 
+			if(foo > 0 ) {
+				powM <- floor(log(foo,gtobase)) + 1 # how many places will I need
+				mout <- rep('0',powM+1) 
+			} else{
+				powM = 0
+				mout <- rep('0',times = max(16,binSize)+1)
+			}
 			if(powM >0 ) {	
 		#remember leftmost element of vector is index 1 
 				for (jp in powM:1) {
@@ -222,35 +226,34 @@ if (!length(powM) || !is.numeric(powM) || powM < 0) browser()
 # just start with:
 # CHANGE Jan 24: split so all isPos get handled either in binary code or in all other bases. 
 #	if(isPos && tobase == 2){
-#	browser()
 	if(isPos) {
 		if(tobase == 2){
-		ktmp <- allChar[rev(as.numeric(mout)+1)]
-		klen <- length(ktmp)
-		if(outTwosComp && ktmp[1] == '1'){
-			ktmp <- c('0',ktmp)
+			ktmp <- allChar[rev(as.numeric(mout)+1)]
 			klen <- length(ktmp)
-		}
+			if(outTwosComp && ktmp[1] == '1'){
+				ktmp <- c('0',ktmp)
+				klen <- length(ktmp)
+			}
 # now expand to binsize
-		ktmp <- c(rep('0',times = max(0,binSize - klen)), ktmp)
-		klen <- length(ktmp)
-# need this for when binSize is zero
-		ktmp <- c(rep('0',times = 3 -(3+klen)%%4),ktmp)
-		theans[[jl]] <- paste0(c(ktmp) ,sep='',collapse='')
-	} else {
+			ktmp <- c(rep('0',times = max(0,binSize - klen)), ktmp)
+			klen <- length(ktmp)
+	# need this for when binSize is zero
+			ktmp <- c(rep('0',times = 3 -(3+klen)%%4),ktmp)
+			theans[[jl]] <- paste0(c(ktmp) ,sep='',collapse='')
+		} else {
 # all other bases, positive value
-#  stick in binSize fix here
-			jltmp <- paste0(allChar[rev(as.numeric(mout)+1)],sep='',collapse='')
+#   don't want to collapse jltmp until after adding the zeros
+			jltmp <- allChar[rev(as.numeric(mout)+1)]
 			jlen <- length(jltmp)
 			jltmp <- c(rep('0',times = max(0,binSize - jlen)), jltmp)
-			theans[[jl]] <- jltmp 
-#   paste0(allChar[rev(as.numeric(mout)+1)],sep='',collapse='')
+# June 2025: this was a bug I missed -- forgot to collapse it
+#			theans[[jl]] <- jltmp 
+			theans[[jl]] <- paste0(jltmp ,sep = '', collapse='')
 		}	#end of if else tobase==2
 	}  else {
 # it's !isPos 
 # remember outTwosComp is forced to FALSE if tobase != 2		
-#	 if (!isPos) {
-		theans[[jl]] <- paste0(allChar[rev(as.numeric(mout)+1)],sep='',collapse='')
+			theans[[jl]] <- paste0(allChar[rev(as.numeric(mout)+1)],sep='',collapse='')
 # I made xchar stuff positive for 2s comp inputs
 # But I didn't make theans[[]] include the lead zero
 			jltmp <- unlist(strsplit(theans[[jl]][1],'') )
@@ -283,10 +286,11 @@ if (!length(powM) || !is.numeric(powM) || powM < 0) browser()
 					}
 #'move' the existing lead '1'to the lead position
 					jltmp <- c(jltmp[1], rep('1',times=max(0,binSize-blen)), jltmp[2:blen])	
-				} #end of first 'else' 	
+				} #end of if else 	as.numeric theans
 				theans[[jl]] <- paste0(c(jltmp), sep='', collapse='')					
 #	this else follows if(tobase == 2 && outTwosComp)
 			} else {
+	# i.e. not both tobase ==2 AND outTwoscomp true
 # now expand to binSize
 				if(tobase == 2) {
 					jlen <- length(jltmp)
@@ -296,31 +300,31 @@ if (!length(powM) || !is.numeric(powM) || powM < 0) browser()
 					jltmp <- c(rep('0',times = 3 -(3+jlen)%%4),jltmp)	
 					theans[[jl]] <- paste0(c(jltmp), sep='', collapse='')
 				}else {
-#  new thing to allow sticking zeros in front of any base.	
+#   stick zeros in front of any other  base.	
 					jlen <- length(jltmp)
 					jltmp <- c(rep('0',times = max(0,binSize - jlen)), jltmp)
 					theans[[jl]] <- paste0(jltmp,sep='',collapse='')
-			}
+					}
 				theans[[jl]] <- paste0(c('-',theans[[jl]]), sep='', collapse='')
 			}  #end of if tobase ==2 else	
 				
 		} # end of the "else" for !isPos
 #	} # end of if isPos
 } #end of for jl
-# as.bigz thinks a lead '0' means octal, so to be on the safe side
-# strip lead zeros 
-# but if I feed it just plain '00' this may lead to a null argument
-# browser()
+# as.bigz thinks a lead '0' means octal, so to be  safe  strip lead zeros 
 if (tobase == 10 ) {
 	switch(classOut[1], 
 		'numeric'= for(jj in 1:length(theans)) theans[[jj]] <- as.numeric(theans[[jj]]),
 		'bigz'= {	
-#if theans is just '0' this deletes it. So  skip single-char inputs			
+#if theans is just '0' this deletes it. So  skip single-char inputs	
+#  extend '0' after this.  the output is "" , which is hard to identify		
 			for(jj in 1:length(theans) ){
 				if (nchar(theans[[jj]]) > 1){
 					theans[[jj]]  <- gsub('^0{1,}', '', theans[[jj]])
 					theans[[jj]]  <- gsub('^-0{1,}', '-', theans[[jj]])
 				}
+# new fix
+				if(!nchar(theans[jj])) theans[[jj]] <- paste0(rep('0', times= max(16,binSize)), sep='', collapse = '')			
 				theans[[jj]] <- gmp::as.bigz(theans[[jj]])
 			} 
 		},
@@ -328,10 +332,13 @@ if (tobase == 10 ) {
 # thinks I fed it an octal.		
 		'mpfr'=  {
 			for(jj in 1:length(theans) ) {
-										if (nchar(theans[[jj]]) > 1){
+				if (nchar(theans[[jj]]) > 1){
 					theans[[jj]]  <- gsub('^0{1,}', '', theans[[jj]])
 					theans[[jj]]  <- gsub('^-0{1,}', '-', theans[[jj]])
 					}
+# new fix
+				if(!nchar(theans[jj])) theans[[jj]] <- paste0(rep('0', times=max(16,binSize)), sep='', collapse = '')			
+
 			 theans[[jj]] <- Rmpfr::.bigz2mpfr(gmp::as.bigz(theans[[jj]]))
 			 }
 		},
