@@ -1,11 +1,33 @@
+# revisions
+# Jun 2023, allow bigz and bigq as well 
+# Jun 2024 fixed bug that fouled up converting binary decimal parts. 
+
 #TODO
-# allow mpfr inputs. use formatDec(x) to convert to char string
+# DUNallow mpfr inputs. use formatDec(x) to convert to char string
 
 # Convert ONLY the fractional part, throwing away any integer part. Live with it. 
  fracB2B <- function( x,  inBase = 10, outBase = 16, maxdig = 0) {
 # input validation
-if(is(x[[1]], 'mpfr') )  x <- formatDec(x)
-if (!is(x[[1]], 'character') && !is(x[[1]],'numeric')) stop('Illegal input class. Use character strings, mpfrs, or numerics.')
+# note- formatDec currently fails for super-high precbits values. 
+# I could hack it here to check precbits(x) and reduces to 1000 to make it work.
+# (check for max(getPrec(x)))
+# formatDec sticks in an "Ncharacter" class which confuses stuff later. Kill it. 
+#  browser()
+# I think this can put a space char that I never want, so ,clean that out too
+if(is(x[[1]], 'mpfr') ) {
+	x <- as.character(formatDec(x))
+	x <- gsub(' ','',x)
+}
+ 
+# new
+if(is(x[[1]],'bigz')) return(0)  #no fraction
+if(is(x[[1]], 'bigq')) {
+	x <- as.character(Rmpfr::formatDec(Rmpfr::.bigq2mpfr(x)) )
+	x <- gsub(' ','',x)
+}
+# clean this up
+#if (!is(x[[1]], 'character') && !is(x[[1]],'numeric')) stop('Illegal input class. Use character strings, mpfrs, or numerics.')
+if (!class(x[[1]]) %in% c('mpfr','numeric','character')) stop(c("Unknown input class" , class(x[[1]]) ) )
 if(!(2<=inBase && 36 >= inBase && 2<=outBase && 36>= outBase)){
 	stop('Both bases must be in range 2:36')
 	}
@@ -34,6 +56,7 @@ for(jx in 1:length(x)) 	{
 		next
 	}
 # remove any integer part 
+# browser()
 	 x[[jx]] <- gsub(paste0('^.{0,}[',thedec,']'),'',x[[jx]] )
 	tmpx <- base2base(x[[jx]], inBase, 10, classOut = 'character')[[1]]
 #browser()
@@ -42,6 +65,7 @@ for(jx in 1:length(x)) 	{
 		theans[[jx]] <- tmpx
 		next
 	}
+# TODO: see if I want to remove lead zeros from x[[jx]] . probably not
 	thepow <- nchar(x[[jx]]) # yes, watch the input size
 # now convert  to a bigq
 # since integer part was removed, just divide by "shift power" of input
@@ -55,6 +79,7 @@ for(jx in 1:length(x)) 	{
 	loopcount = 0
 	domore = TRUE
 	theans[[jx]] <- as.character(NULL)
+# browser()
 	while(domore && loopcount < maxdig){		
 		tmp <- tmpfrac * outz # the "new" integer part will be appended ..
 		tmpint <- as.bigz(tmp)
@@ -64,11 +89,22 @@ for(jx in 1:length(x)) 	{
 		loopcount = loopcount + 1
 	}
 # Convert each term separately to ensure, e.g., 14 base 10 turns into E base 16 
-	termtmp <- rep('0',times=length(theans[[jx]]))	
-	for (jz in 1:length(theans[[jx]])){
-		termtmp[jz] <- unlist(base2base(theans[[jx]][[jz]], 10, outBase, classOut ='character'))
-	}
+# seems to be a bug here - 0.2 decimal input is giving way too many zeros
+# to the output binary representation.  All other bases seem ok,
+# and converting FROM base2 seems ok .
+# this is because base2base forces base2 out, when character, to 4N slots
+# I need to figure out how to keep 'wanted' lead zeros ? 
+# Since binary is a special case, and its REAL SIMPLE, just skip the call!  
+# new:
+	if(outBase == 2) {
+		theans[[jx]] <- paste0(theans[[jx]], collapse = '')
+	} else {
+		termtmp <- rep('0',times=length(theans[[jx]]))	
+		for (jz in 1:length(theans[[jx]])){
+			termtmp[jz] <- unlist(base2base(theans[[jx]][[jz]], 10, outBase, classOut ='character'))
+		}
 	theans[[jx]] <- paste0(termtmp,collapse='') 
+	}
 # collapse that list - and put decimal back in, ISO std
 	theans[[jx]] <- paste0(c('0', thedec, unlist(theans[[jx]]), collapse='') )
 	theans[[jx]] <- paste0(c(negval[[jx]],theans[[jx]]),collapse='')
